searchNodes=[{"doc":"A pure-Elixir implementation of statecharts inspired by David Harel's Statecharts: a visual formalism for complex systems paper David Khourshid's JavaScript XState library Installation This package can be installed by adding statechart to your list of dependencies in mix.exs : def deps do [ { :statechart , &quot;~&gt; 0.1.0&quot; } ] end Concepts We'll model a simple traffic light to illustrate some statechart concepts. This &quot;machine&quot; defaults to the off state (that's what the dot-arrow signifies). If we then send the machine a TOGGLE event, it transitions to the on state. From there, it automatically drops into the red state (again, because of the dot-arrow). At this point, the machine is in both the on and red states. If we send it a NEXT event, we transition to the green state (which you can also think of as the on/green state). Another NEXT event, and we transition to the yellow state. In this way, the light will just keep cycling through the colors. If we send it a TOGGLE at this point, it will transition back to off . If we now send the machine a NEXT event (while it's in the off state), nothing happens. Usage There are three steps to modeling via the Statechart library: DEFINE statechart/2 state/3 &gt;&gt;&gt;/2 INSTANTIATE MyStatechart.new/0 MANIPULATE trigger/2 states/1 in_state?/2 last_event_status/1 We'll model the above traffic light using these three steps. DEFINE defmodule TrafficLight do use Statechart statechart default : :off do state :off do :TOGGLE &gt;&gt;&gt; :on end state :on , default : :red do :TOGGLE &gt;&gt;&gt; :off state :red , do : :NEXT &gt;&gt;&gt; :green state :yellow , do : :NEXT &gt;&gt;&gt; :red state :green , do : :NEXT &gt;&gt;&gt; :yellow end end end INSTANTIATE The module containing your statechart definition automatically has a new/0 function injected into it. traffic_light = TrafficLight . new ( ) It returns you a statechart/0 struct that you then pass to all the 'MANIPULATE' functions. MANIPULATE The machine starts in the off state: [ :off ] = Statechart . states ( traffic_light ) true = Statechart . in_state? ( traffic_light , :off ) false = Statechart . in_state? ( traffic_light , :on ) Send it a NEXT event without it being on yet: traffic_light = Statechart . trigger ( traffic_light , :NEXT ) # Still off... true = Statechart . in_state? ( traffic_light , :off ) # ...but we can see that the last event wasn&#39;t valid: :error = Statechart . last_event_status ( traffic_light ) Let's turn it on: traffic_light = Statechart . trigger ( traffic_light , :TOGGLE ) [ :on , :red ] = Statechart . states ( traffic_light ) true = Statechart . in_state? ( traffic_light , :on ) true = Statechart . in_state? ( traffic_light , :red ) false = Statechart . in_state? ( traffic_light , :off ) false = Statechart . in_state? ( traffic_light , :green ) Now the NEXT events will have an effect: traffic_light = Statechart . trigger ( traffic_light , :NEXT ) [ :on , :green ] = Statechart . states ( traffic_light ) Error-checking Statechart has robust compile-time checking. For example, compiling this module will result in a StatechartError at the state :on line. defmodule ToggleStatechart do use Statechart statechart default : :on do # Whoops! We&#39;ve misspelled &quot;off&quot;: state :on , do : :TOGGLE &gt;&gt;&gt; :of state :off , do : :TOGGLE &gt;&gt;&gt; :on end end Other statechart / state machine libraries With a plethora of other related libraries, why did we need another one? I wanted one that had very strict compile-time checks and a simple DSL. Other libraries you might look into: Machinery as_fsm gen_statem https://github.com/sasa1977/fsm","ref":"Statechart.html","title":"Statechart","type":"module"},{"doc":"Register a transtion from an event and target state. StatechartError raised when... event is non-atom event occurs elsewhere amongst this node's ancestors or descendents target_state doesn't exist &gt;&gt;&gt;/2 is called outside of a state block","ref":"Statechart.html#%3E%3E%3E/2","title":"Statechart.>>>/2","type":"macro"},{"doc":"Determine if the given state is in the given compound state","ref":"Statechart.html#in_state?/2","title":"Statechart.in_state?/2","type":"function"},{"doc":"Returns :ok is last event was valid and caused a transition","ref":"Statechart.html#last_event_status/1","title":"Statechart.last_event_status/1","type":"function"},{"doc":"Create a statechart node. name must be an atom and must be unique amongst nodes defined in this module's statechart. The way to have multiple nodes sharing the same name is to define statechart partials in separate module and then insert those partials into a parent statechart. StatechartError raised when... name is non-atom name is non-unique (another node already has the same name) assigning a default to a leaf node a default targets a non-descendent state/2 is called outside of a statechart block","ref":"Statechart.html#state/3","title":"Statechart.state/3","type":"macro"},{"doc":"Create and register a statechart to this module. May only be used once per module. defmodule ToggleStatechart do use Statechart statechart do state :on , default : true , do : :TOGGLE &gt;&gt;&gt; :off state :off , do : :TOGGLE &gt;&gt;&gt; :on end end statechart/2 accepts a :module option. In the below example, the module containing the statechart is Toggle.Statechart defmodule Toggle do use Statechart statechart module : Statechart do state :on , default : true , do : :TOGGLE &gt;&gt;&gt; :off state :off , do : :TOGGLE &gt;&gt;&gt; :on end end In this way, many statecharts may be declared easily in one file: defmodule MyApp.Statechart do use Statechart # module: MyApp.Statechart.Toggle statechart module : Toggle do state :on , default : true , do : :TOGGLE &gt;&gt;&gt; :off state :off , do : :TOGGLE &gt;&gt;&gt; :on end # module: MyApp.Statechart.Switch statechart module : Switch do state :on , default : true , do : :SWITCH_OFF &gt;&gt;&gt; :off state :off , do : :SWITCH_ON &gt;&gt;&gt; :on end end StatechartError raised when... statechart/2 is used more than once per module","ref":"Statechart.html#statechart/2","title":"Statechart.statechart/2","type":"macro"},{"doc":"Get the current compound state","ref":"Statechart.html#states/1","title":"Statechart.states/1","type":"function"},{"doc":"Send an event to the statechart","ref":"Statechart.html#trigger/2","title":"Statechart.trigger/2","type":"function"},{"doc":"","ref":"Statechart.html#t:event/0","title":"Statechart.event/0","type":"type"},{"doc":"","ref":"Statechart.html#t:state/0","title":"Statechart.state/0","type":"type"},{"doc":"","ref":"Statechart.html#t:statechart/0","title":"Statechart.statechart/0","type":"opaque"},{"doc":"Raised at compile time when anything inside Statechart.statechart/2 fails validation. See Statechart macros for details.","ref":"StatechartError.html","title":"StatechartError","type":"exception"},{"doc":"All notable changes to this project will be documented in this file. The format is based on Keep a Changelog , and this project adheres to Semantic Versioning .","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"Added Statechart.&gt;&gt;&gt;/2 Statechart.state/3 Statechart.statechart/2 Statechart.in_state?/2 Statechart.last_event_status/1 Statechart.states/1 Statechart.trigger/2","ref":"changelog.html#0-1-0-2022-11-06","title":"Changelog - 0.1.0 - 2022-11-06","type":"extras"}]